# arcticspas
A low-level client library for accessing Arctic Spas API.
Generated from the [OpenAPI definition](https://api.myarcticspa.com/docs/arctic-spas-openapi.json)
using [openapi-python-client](https://pypi.org/project/openapi-python-client/).

Note: for generic use, please consider the [pyarcticspas](https://pypi.org/project/pyarcticspas) library.

## Usage
First, create a client:

```python
from arcticspas import Client

token = "(Token received from https://myarcticspa.com/spa/SpaAPIManagement.aspx)"
client = Client(base_url="https://api.myarcticspa.com", headers={"X-API-KEY": token})
```

Now call your endpoint and use your models:

```python
from arcticspas.models import V2SpaResponse200
from arcticspas.api.spa_control import v2_spa
from arcticspas.types import Response

with client as client:
    status: V2SpaResponse200 = v2_spa.sync(client=client)
    # or if you need more info (e.g. status_code)
    response: Response[V2SpaResponse200] = v2_spa.sync_detailed(client=client)
```

Or do the same thing with an async version:

```python
from arcticspas.models import V2SpaResponse200
from arcticspas.api.spa_control import v2_spa
from arcticspas.types import Response

async with client as client:
    status: V2SpaResponse200 = await v2_spa.asyncio(client=client)
    response: Response[V2SpaResponse200] = await v2_spa.asyncio_detailed(client=client)
```

By default, when you're calling an HTTPS API it will attempt to verify that SSL is working correctly. Using certificate verification is highly recommended most of the time, but sometimes you may need to authenticate to a server (especially an internal server) using a custom certificate bundle.

```python
client = AuthenticatedClient(
    base_url="https://api.example.com", 
    token="SuperSecretToken",
    verify_ssl="/path/to/certificate_bundle.pem",
)
```

You can also disable certificate validation altogether, but beware that **this is a security risk**.

```python
client = AuthenticatedClient(
    base_url="https://api.example.com", 
    token="SuperSecretToken", 
    verify_ssl=False
)
```

Things to know:
1. Every path/method combo becomes a Python module with four functions:
    1. `sync`: Blocking request that returns parsed data (if successful) or `None`
    1. `sync_detailed`: Blocking request that always returns a `Request`, optionally with `parsed` set if the request was successful.
    1. `asyncio`: Like `sync` but async instead of blocking
    1. `asyncio_detailed`: Like `sync_detailed` but async instead of blocking

1. All path/query params, and bodies become method arguments.
1. If your endpoint had any tags on it, the first tag will be used as a module name for the function (my_tag above)
1. Any endpoint which did not have a tag will be in `arcticspas.api.default`

## Advanced customizations

There are more settings on the generated `Client` class which let you control more runtime behavior, check out the docstring on that class for more info. You can also customize the underlying `httpx.Client` or `httpx.AsyncClient` (depending on your use-case):

```python
from arcticspas import Client

def log_request(request):
    print(f"Request event hook: {request.method} {request.url} - Waiting for response")

def log_response(response):
    request = response.request
    print(f"Response event hook: {request.method} {request.url} - Status {response.status_code}")

client = Client(
    base_url="https://api.example.com",
    httpx_args={"event_hooks": {"request": [log_request], "response": [log_response]}},
)

# Or get the underlying httpx client to modify directly with client.get_httpx_client() or client.get_async_httpx_client()
```

You can even set the httpx client directly, but beware that this will override any existing settings (e.g., base_url):

```python
import httpx
from arcticspas import Client

client = Client(
    base_url="https://api.example.com",
)
# Note that base_url needs to be re-set, as would any shared cookies, headers, etc.
client.set_httpx_client(httpx.Client(base_url="https://api.example.com", proxies="http://localhost:8030"))
```

